#
# Copyright (c) 2002, Jason Erickson <jerickso@stickpeople.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of Jason Erickson nor the names of its contributors
#   may be used to endorse or promote products derived from this software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.



from wxPython.wx import *
import string

import vimThread
import GUICommon


# Thread Interfaces
#   select(filename, line #)
#   start()
#   stop()
#   isRunning()
#   remove()
#   insert()
#
# GUI Interfaces
#   fileSelected(filename, line#)
#   insert(filename)
#   remove(filename)
#
# VIM Interface
#   select(filename, line#)
#   remove(filename)
#   insert(filename)
#   files()
#
#  -----------------
#  |     GUI       |
#  -----------------
#       |      ^
#       V      |
#  -----------------
#  |    THREAD     |
#  -----------------
#       |      ^
#       V      |
#  -----------------
#  |      VIM      |
#  -----------------

wxEVT_ADD_BUTTON = 26000
wxEVT_REMOVE_BUTTON = 26001
wxEVT_NEW_FILE_SELECTED = 26002


def EVT_ADD_BUTTON(win, func):
  win.Connect(-1, -1, wxEVT_ADD_BUTTON, func)

def EVT_REMOVE_BUTTON(win, func):
  win.Connect(-1, -1, wxEVT_REMOVE_BUTTON, func)

def EVT_NEW_FILE_SELECTED(win, func):
  win.Connect(-1, -1, wxEVT_NEW_FILE_SELECTED, func)


class RemoveButton(wxPyEvent):
  def __init__(self, text):
    wxPyEvent.__init__(self)
    self.SetEventType(wxEVT_REMOVE_BUTTON)
    self.text = text

class AddButton(wxPyEvent):
  def __init__(self, text):
    wxPyEvent.__init__(self)
    self.SetEventType(wxEVT_ADD_BUTTON)
    self.text = text

class FileSelected(wxPyEvent):
  def __init__(self, text):
    wxPyEvent.__init__(self)
    self.SetEventType(wxEVT_NEW_FILE_SELECTED)
    self.text = text



class wxGUI(wxFrame):
  def __init__(self, parent, ID, title, queue, files):
      wxFrame.__init__(self, parent, ID, title, wxDefaultPosition)
      self.parent = parent

      self.sizerVert = wxBoxSizer(wxVERTICAL)
      self.sizerVert.Fit(self)

      self.SetAutoLayout(true)
      self.SetSizer(self.sizerVert)

      self.sizerHor = []
#      self.rows = []

      # Dictionary that has the file strings associated with the ids
      self.buttonKeeper = GUICommon.buttonKeeper()

      self.currentlySelected = None

      # Setup the events for Insert and Remove
      EVT_ADD_BUTTON(self, self.onInsert)
      EVT_REMOVE_BUTTON(self, self.onRemove)
      EVT_NEW_FILE_SELECTED(self, self.onNewFile)

      # Setup the events for Closing the Window
      EVT_CLOSE(self, self.OnCloseWindow)

      # Initialize the threader
      self.threader = vimThread.pollThread(self, queue)
      self.threader.start()

      # Lets finally get rid of the files that we want loaded in
      for i in files:
        self.threader.insert(i)



  # A request to remove a button
  def remove(self, file):
    wxPostEvent(self, RemoveButton(file))


  # A request to add a button
  def insert(self, file):
    # Post the event for the filename
    wxPostEvent(self, AddButton(file))

  # A request that a new file was selected
  def select(self, file, line=0):
    wxPostEvent(self, FileSelected(file))


  # A request to add a button
  def onInsert(self, evt):
    file = evt.text
    # Get only the file name from the file
    index = string.rfind(file, '\\')
    text = file[index+1:]

    # TODO: Make sure that the name does not exist
    # Store the file name associated with the text button
    button = wxToggleButton(self, -1, text)
    EVT_TOGGLEBUTTON(button, -1, self.onClick)
    button.SetToolTipString(file)
    width, height = button.GetSize()

    self.buttonKeeper.insert(file, text, button, width, None)

    self.updateButtons()


  def updateButtons(self):
    # Remove all buttons from the Horizontal Layouts
    for i in self.buttonKeeper.getKeys():
      if self.buttonKeeper.getOther(i) != None:
        self.buttonKeeper.getOther(i).Remove(self.buttonKeeper.getButton(i))
        self.buttonKeeper.getOther(i).Layout()

    # Remove all 'rows' from the Layout Editor
    for i in range(0, len(self.sizerHor)):
      self.sizerVert.Remove(self.sizerHor[0])
      del self.sizerHor[0]


    # Resize after removal, to get rid of straglers
    self.sizerVert.Layout()
    self.sizerVert.Fit(self)
    self.Layout()


    # Setup the first Horizontal Button bar
    self.sizerHor.append(wxBoxSizer(wxHORIZONTAL))

    width = 0
    maxwidth, height = wxDisplaySize()
    maxwidth = maxwidth - 4

    array = self.buttonKeeper.getKeys()

    # Add Buttons
    # Setup the button
    for i in array:

      # Get the width of the current window to determine if we need to go down
      # If the size is too great, insert on a next row
      bsize = self.buttonKeeper.getWidth(i)
      if width + bsize > maxwidth:
        # Create a new row
        self.sizerHor[len(self.sizerHor)-1].Layout()
        self.sizerVert.Add(self.sizerHor[len(self.sizerHor)-1], 0, wxALIGN_TOP)
        self.sizerHor.append(wxBoxSizer(wxHORIZONTAL))
        width = 0
#        self.rows.append(0)

      width = width + bsize
#      self.rows[len(self.sizerHor)-1] += 1
      self.buttonKeeper.setOther(i, self.sizerHor[len(self.sizerHor)-1])
      self.sizerHor[len(self.sizerHor)-1].Add(self.buttonKeeper.getButton(i), \
                0, wxALIGN_TOP)

    self.sizerHor[len(self.sizerHor)-1].Layout()
    self.sizerVert.Add(self.sizerHor[len(self.sizerHor)-1], 0, wxALIGN_TOP)

    # Resize
    self.sizerVert.Layout()
    self.sizerVert.Fit(self)
    self.Layout()



  def onRemove(self, evt):
    file = evt.text
    # Get only the file name from the file
    index = string.rfind(file, '\\')
    text = file[index+1:]

    for i in self.buttonKeeper.getKeys():
      if self.buttonKeeper.getOther(i) != None:
        self.buttonKeeper.getOther(i).Remove(self.buttonKeeper.getButton(i))

    # Destroy Button
    button = self.buttonKeeper.getButton(file)
    button.Disconnect(-1, wxEVT_COMMAND_BUTTON_CLICKED)
    button.Hide()  # ??? Some strange reason, must Hide before destroying
    self.buttonKeeper.delete(file)
    del button

    # Redraw
    self.updateButtons()


  def onNewFile(self, event):
    file = event.text
    # TODO: Is this a temp solution to the KeyErrors?  There is a race
    #  condition sometimes when a file is dragged and dropped.  The 'active'
    #  file get's sent first before the button data gets created/updated.
    #
    try:
      obj = self.buttonKeeper.getButton(file)

      if self.currentlySelected != None:
        if self.currentlySelected != obj:
          self.currentlySelected.SetValue(0)
      self.currentlySelected = obj
      self.currentlySelected.SetValue(1)
    except KeyError:
      pass


  def onClick(self, event):
    obj = event.GetEventObject()

    if self.currentlySelected != None:
      if self.currentlySelected != obj:
        self.currentlySelected.SetValue(0)
        self.selectFile(obj.GetToolTip().GetTip())
    self.currentlySelected = obj
    self.currentlySelected.SetValue(1)


  def selectFile(self, file, lineNum = 0):
    self.threader.select(file, lineNum)


  def stop(self):
    self.threader.stop()

  def OnCloseWindow(self, evt):
    self.stop()
    self.Destroy()

  def __del__(self):
    self.stop()


def startGUI(queue, files):
   app = wxPySimpleApp()
   app.frame = wxGUI(None, -1, "VIM Control Center", queue, files)
   app.frame.SetPosition((0,0))
   app.frame.Show(true)
   app.SetTopWindow(app.frame)
   app.MainLoop()


class MyApp(wxApp):
  def OnInit(self, queue, files):
    frame = wxGUI(NULL, -1, "VIM Control Center", queue, files)
    frame.SetPosition((0,0))
    frame.Show(true)
    frame.SetSizeHints(-1, -1, 800, 600)
    self.SetTopWindow(frame)
    return wx.TRUE
