#
# Copyright (c) 2002, Jason Erickson <jerickso@stickpeople.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of Jason Erickson nor the names of its contributors
#   may be used to endorse or promote products derived from this software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.


import socket
import threading
import Queue

import GUICommon
import vimThread


# VimMessaging(files):
#    files - an array of files be loaded
#
# This class is a helper class that will do one of two things:
#   1) It will try to determine if an instance of the VIM Command Center
#      is running.  It does this by checking to see if it can bind to a
#      particular socket on the machine.  If it can, then it assumes that
#      there are no other instances of the Command Center running and
#      launches the GUI.
#   2) If it determines that an instance of the VIM Command Center is
#      running, it will pass the files through sockets to program the
#      VIM Command Center that is running.  There is a small security issue
#      with using this method, but I have minimized it by only allowing
#      messages from the local host through.  This could easily be changed
#      to allow for some interesting loading of files from remote machines.
#
#
class VimMessaging:
  HOST = 'localhost'
  PORT = 18975

  def __init__(self, files):
    self.flag = 0
    self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

    # Check to see if port is open, if so, then we send the file name
    try:
      self.socket.bind((self.HOST, self.PORT))
      server = 1

    except socket.error:
      server = 0

    # If we are a server (aka. the first), create a Q, spawn off our listener
    #  thread, and then GUI
    if server == 1:
      self.flag = 1
      # Create the Q
      self.Q = Queue.Queue(0)
      # Spawn off serverThread
      self.thread = threading.Thread(None, self.run, "socketThread")
      self.thread.start()
      # Spawn off GUI, passing the Q to communiate with it.  Don't forget
      #   about the files we need to load.
      gui = GUICommon.GUI(self.Q, files)
      # Once we hit this point, the GUI is shutting down, so start tearing
      #   down.
      self.stop()
    else:
      # The VIM Control Center Exists, lets connect through sockets and
      #   pass the files names over
      self.socket.close()
      for i in files:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((self.HOST, self.PORT))
        s.send(i)
        s.close()


  def run(self):
    while self.flag == 1:
      self.socket.listen(1)
      conn, addr = self.socket.accept()
      data = conn.recv(1024)
      conn.close()

      # TODO: Should we do a sanity check here in regards to the file being
      #   passed in?  Can we?

      if self.flag == 1:
        # Pass the message into the queue, only if the request is from our
        #    machine.
        if addr == '127.0.0.1':
          self.Q.put(data)

  def stop(self):
    # Only do this if you are the server.  We need to send a message to our
    #   server since it is blocking.  We just send a message that will be
    #   discarded.
    if self.flag == 1:
      # Set the global flag to kick it out
      self.flag = 0
      # Send some data to the port
      s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
      s.connect((self.HOST, self.PORT))
      s.send("Get Me outa here, stat")
      s.close()

  def __del__(self):
    self.stop()

